// Package
package zeroslackr.installer;

// Imports
import java.net.*;
import java.io.*;
import org.w3c.tools.crypt.Md5;

/**
 * Abstract Downloader Class.
 * <p>
 * Methods for downloading files.
 * <p>
 * Licensed under <A HREF="http://www.gnu.org/licenses/gpl.txt">GNU GPL (Version 3)</A>. 
 * <p>
 * Sources: 
 * <A HREF="http://java.sun.com/j2se/1.4.2/docs/api/java/net/URL.html">Java's URL class API</A> 
 * <p>
 * @author	Keripo
 * @version	Jan 18, 2008
 */

public abstract class Downloader {
	
	/**
	 * Download a file to a location.
	 * @param	url	String representing URL to download from.
	 * @param	dest	Destination to download to.
	 */
	public static void download(String url, String dest)
		throws MalformedURLException, IOException {
		
		// Create directories if needed
		new File(dest).mkdirs();
		
		// Create streams
		InputStream in = new URL(url).openStream();
		FileOutputStream out
			= new FileOutputStream(Tools.convertPath(dest + "/" + getFileName(url)));
		
		// Write to file
		byte[] buffer = new byte[256]; // Arbitrary length
		int len;
        while ( ( len = in.read(buffer) ) != -1 ) {
        	out.write(buffer, 0, len);
        }
        
        // Close
		out.close();
		in.close();
		
	} // download
	
	/**
	 * Get file name in URL.
	 * @param	url	String representing URL to check.
	 * @return	Name of file.
	 */
	public static String getFileName(String url) {
		return url.substring(url.lastIndexOf('/')+1);
	} // getName
	
	/**
	 * Get the basename of a file (ie without the extension).
	 * @param	file	String representing URL to check.
	 * @return	Basename of file.
	 */
	public static String getBaseName(String file) {
		return file.substring(file.lastIndexOf('/')+1, file.lastIndexOf('.'));
	} // getBaseName

	/**
	 * Calculates the Md5 hash of a target file.
	 * @param	file	String of path to file.
	 * @return	String of Md5 hash.
	 */
	public static String getMd5 (String file) throws FileNotFoundException, IOException {;
		Md5 hash = new Md5 ( new FileInputStream (file) );
		hash.getDigest();
		return hash.getStringDigest();
	} // getMd5
	
} // Downloader class
