// Package
package zeroslackr.installer;

// Imports
import java.io.*;
import java.util.*;

/**
 * Main Installer.
 * <p>
 * Where it all starts. See the printError method for exit values.
 * <p>
 * TODO list:<br>
 * - figure out how to get Eclipse to auto-add referenced<br>
 *   libraries class files when exporting to a JAR<br>
 * - test on Macs and Linux hosts<br>
 * - specialize actions for the different list types<br>
 * - use loader2.bin file from downloaded pack<br>
 * - create an packlist for official iPL downloads<br>
 * - support 7zip archives via J7Zip library (which is still in development)<br>
 * - expand Pack object (not sure if needed though)<br>
 * - add music extraction function via <A HREF="http://javamusictag.sourceforge.net/">Java ID3 Tag Library</A><br>
 * - add GUI (for new users but not necessary)<br>
 * - add in MP3 playback functionality (only with GUI)<br>
 * <p>
 * Licensed under <A HREF="http://www.gnu.org/licenses/gpl.txt">GNU GPL (Version 3)</A>. 
 * <p>
 * @see #printError(String error, int errorLevel, Exception e)
 * @author	Keripo
 * @version	Jan 18, 2008
 */

public class Main {
	
	// Install modes
	// Note that Rockbox is disabled for now.
		// Simple
			private static boolean help, backup, restore, loader2;
		// Install
			//static boolean rockbox;
			private static boolean ipodlinux, zeroslackr;
		// Extras
			private static boolean auto, lazy, mirror;
		
	// Folder Structure
		private static final String FILES_FOLDER = "Files";
		private static final String BACKUP_FOLDER = FILES_FOLDER + "/Backup";
		private static final String DOWNLOAD_FOLDER = FILES_FOLDER + "/Download";
		private static final String TOOLS_FOLDER = "Tools";
		private static final String IPODPATCHER_FOLDER = TOOLS_FOLDER + "/iPodPatcher";
		private static final String LOADER2_FOLDER = TOOLS_FOLDER + "/Loader2";
		
		private static final String[] FOLDER_STRUCTURE = {
			FILES_FOLDER, BACKUP_FOLDER, LOADER2_FOLDER, DOWNLOAD_FOLDER,
			TOOLS_FOLDER, IPODPATCHER_FOLDER
		};
		
	// Variables
		private static String ipod;
		private static String packlist;
		private static iPodPatcher ipp;
		private static String fw_backup, loader2_bin;
		private static Scanner in = new Scanner(System.in);
	
	// Defaults
		private static final String IPL_PACKLIST
		= "http://waldo.jworks.ca/iPL/zeroslackr.ini"; // For now
		private static final String ZS_PACKLIST
		= "http://waldo.jworks.ca/iPL/zeroslackr.ini";
		private static final String DEFAULT_FW_BACKUP
			= BACKUP_FOLDER + "/fw_backup.img"; 
		private static final String DEFAULT_LOADER2_BIN
			= LOADER2_FOLDER + "/loader.bin";
		
	// Testing
		//private static String[] testArgs = {"-zs"};
	
	/**
	 * Main installer. Use "zeroslackr-lazy-installer --help" for commands
	 */
	public static void main(String[] args) {
		
		// Initiation
			printSplash();
			// For testing
			if (args.length > 0) {
				checkArgs(args);
			//if (testArgs.length > 0) {
				//checkArgs(testArgs);
			} else {
				lazy = true;
			}
			
		// Setup
			if (help) {
				printCommands();
				System.exit(0);
			}
			createFolderStructure();
			try {
				ipod = Tools.getiPodPath();
			} catch (Exception e) {
				printError("Initiation error.", 2, e);
			}
			if (lazy) {
				System.out.println("Doing a \"--lazy\" install; using all defaults.\n");
				setupLazyInstall();
			}
			
		// Setup iPodPatcher
			if (backup || restore || loader2) {
				setupiPodPatcher();
			}
			
		// Backup/restore
			if (backup) {
				System.out.print("> Backing up iPod's firmware to \"" + fw_backup + "\"... ");
				backup(fw_backup);
				System.out.println("Done.");
			} else if (restore) {
				System.out.print("> Restore iPod's firmware from backup \"" + fw_backup + "\"... ");
				restore(fw_backup);
				System.out.println("Done.");
			}
			
		// Loader 2
			if (loader2) {
				System.out.print("> Installing Loader2 from \"" + loader2_bin + "\"... ");
				installLoader2(loader2_bin);
				System.out.println("Done.");
			}
		
		// Install packs			
			if (packlist != null) {
				System.out.println("> Starting Installation process... ");
				processList(packlist);
			}
			
		// Finish
			System.out.println(
					"> Installation process finished.\n" +
					"   Enjoy!\n" +
					"  ~Keripo");
			in.close();
			System.exit(0);
		
	} // main
	
	/**
	 * Easy method for printing errors. <p>
	 * Error levels:<br>
	 * -1 = warning (non-fatal),<br>
	 *  0 = normal exit,<br>
	 *  1 = bad command,<br>
	 *  2 = initiation error,<br>
	 *  3 = ipodpatcher error,<br>
	 *  4 = packlist error<br>
	 */
	private static void printError(String error, int errorLevel, Exception e) {
		if (errorLevel == -1) {
			System.err.println("Warning: " + error);
		} else {
			System.err.println("Error: " + error);
		}
		if (e != null) {
			System.err.println("Exception: " + e.getMessage());
		}
		if (errorLevel != -1) {
			System.exit(errorLevel);
		}
	} // printError
	
	/**
	 * Print splash text.
	 */
	private static void printSplash() {
		System.out.println(
			"ZeroSlackr Lazy Installer v1.0 - (C) Keripo, 2007\n\n" +
			"This is free software; see the source for copying conditions. There is NO \n" +
			"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
		);
	} // printSplash
	
	/**
	 * Print commands.
	 */
	private static void printCommands() {
		System.out.println(
			"Usage: java -jar zeroslackr-lazy-installer.jar [OPTIONS]\n" +
			"\n" +
			"Where [OPTIONS] is one of the following options:\n" +
			"\n Simple options:\n" +
			" (Note: [FILE] is an optional argument; default settings will be used otherwise.\n" +
			"  -h,	--help			print out commandlist\n" +
			"  -L,	--loader2 [FILE]	re/install Loader2\n" +
			"  -B,	--backup [FILE]		backup iPod firmware\n" +
			"  -R,	--restore [FILE]	restore iPod firmware from backup\n" +
			"\n Install options:\n" +
			//"  -rbx,	--rockbox	install Rockbox\n" +
			"  -ipl,	--ipodlinux		install iPodLinux (standard)\n" +
			"  -zs,	--zeroslackr		install ZeroSlackr (replaces iPL)\n" +
			" (note that you will still need -L if you don't have Loader2)\n" +
			"\n Other options:\n" +
			"  -a,	--auto			auto-yes for all iPodLinux/ZeroSlackr packs\n" +
			//"  -lzy,	--lazy		same thing as \"-L -B -rbx -zs -a\"\n" +
			"  -lzy,	--lazy			same thing as \"-L -B -zs -a\"\n" +
			"  -m,	--mirror [FILE/URL]	load packlist from a mirror\n" +
			"\n");
	} // printCommands	
	
	/**
	 * Create folder structure.
	 */
	private static void createFolderStructure() {
		for (String s: FOLDER_STRUCTURE) {
			File f = new File(Tools.convertPath(s));
			f.mkdirs();
			if (!f.exists()) {
				printError("Unable to create folder \"" + s + "\".", -1, null);
			}
		}
	} // createFolderStructure
	
	/**
	 * Check passed arguments.
	 */
	private static void checkArgs(String[] args) {
		
		// Loop
		int end = args.length - 1;
		for (int i = 0; i < end+1; i++) {
			// Simple
			if (args[i].equals("-h") || args[i].equals("--help")) {
				help = true;
			} else if (args[i].equals("-L") || args[i].equals("--loader2")) {
				loader2 = true;
				if (!(i+1 > end) && args[i+1].charAt(0) != '-') {
					loader2_bin = args[i+1];
					i++;
				} else {
					loader2_bin = DEFAULT_LOADER2_BIN;
				}
			} else if (args[i].equals("-B") || args[i].equals("--backup")) {
				backup = true;
				if (!(i+1 > end) && args[i+1].charAt(0) != '-') {
					fw_backup = args[i+1];
					i++;
				} else {
					fw_backup = DEFAULT_FW_BACKUP;
				}
			} else if (args[i].equals("-R") || args[i].equals("--restore")) {
				restore = true;
				if (!(i+1 > end) && args[i+1].charAt(0) != '-') {
					fw_backup = args[i+1];
					i++;
				} else {
					fw_backup = DEFAULT_FW_BACKUP; 
				}
			// Install
			//} else if (args[i].equals("-rbx") || args[i].equals("--rockbox")) {
				//rockbox = true;
			} else if (args[i].equals("-ipl") || args[i].equals("--ipodlinux")) {
				ipodlinux = true;
				packlist = IPL_PACKLIST;
			} else if (args[i].equals("-zs") || args[i].equals("--zeroslackr")) {
				zeroslackr = true;
				packlist = ZS_PACKLIST;
			// Other
			} else if (args[i].equals("-a") || args[i].equals("--auto")) {
				auto = true;
			} else if (args[i].equals("-lzy") || args[i].equals("--lazy")) {
				lazy = true;
			} else if (args[i].equals("-m") || args[i].equals("--mirror")) {
				mirror = true;
				if (!(i+1 > end) && args[i+1].charAt(0) != '-') {
					packlist = args[i+1]; 
					i++;
				}
			// Unknown
			} else {
				printError("Unknown command.", 1, null);
			}
		} // Loop
		
		// Checks
		if (backup && restore) {
			printError("There's no point in both backing up and restoring!", 1, null);
		}
		if (ipodlinux && zeroslackr) {
			printError("Choose either the standard iPodLinux installation via \"-ipl\"\n" +
				"or the customized iPodLinux installation with ZeroSlackr via \"-zs\".", 1, null);
		}
		if (mirror && (ipodlinux || zeroslackr)) {
			printError("Do not use \"-m\" in combination with " +
					"\"-ipl\" or \"-zs\".", 1, null);
		}
		if (auto && !ipodlinux && !zeroslackr) {
			printError("The \"-a\" command is for use with \"-ipl\" or \"-zs\".", 1, null);
		}
		
	} // checkArgs

	/**
	 * Setup lazy install mode
	 */
	private static void setupLazyInstall() {
		loader2 = true;
		backup = true;
		zeroslackr = true;
		auto = true;
		fw_backup = DEFAULT_FW_BACKUP;
		loader2_bin = DEFAULT_LOADER2_BIN;
		packlist = ZS_PACKLIST;
	} // setupLazyInstall
	
	/**
	 * Setup iPodPatcher.
	 */
	private static void setupiPodPatcher() {
		ipp = new iPodPatcher(IPODPATCHER_FOLDER);
		if (!ipp.exists()) {
			printError("ipodpatcher binary cannot be found.", 2, null);
		}
	} // setupiPodPatcher
	
	/**
	 * Backup iPod's firmware via ipodpatcher.
	 * @param	path	Path to firmware image.
	 */
	private static void backup(String path) {
		try {
			ipp.backup(path);
		} catch (Exception e) {
			printError("Unable to create a backup.", 3, e);
		}
	} // backup
	
	/**
	 * Restore iPod's firmware from backup via ipodpatcher.
	 * @param	path	Path to firmware image.
	 */
	private static void restore(String path) {
		try {
			ipp.restore(path);
		} catch (Exception e) {
			printError("Unable to restore from backup.", 3, e);
		}
	} // restore
	
	/**
	 * Install Loader2 via ipodpatcher.
	 * @param	path	Path to loader.bin file.
	 */
	private static void installLoader2(String path) {
		try {
			ipp.install(path);
		} catch (Exception e) {
			printError("Unable to install Loader2.", 3, e);
		}
	} // installLoader2
	
	/**
	 * Process list.
	 */
	private static void processList(String listFile) {
		try {
			
			// Setup
				System.out.println("  Processing packlist \"" + listFile + "\":");
				String file = FILES_FOLDER + "/" + Downloader.getFileName(packlist);
				Downloader.download(packlist, FILES_FOLDER);
				List list = new List(file);
			
			// Lists
			// TODO: Specialize actions later.
				System.out.println("  Kernels:");
				for (Pack p: list.kernel) {
					installPack(p);
				}
				System.out.println("  Userland:");
				for (Pack p: list.userland) {
					installPack(p);
				}
				System.out.println("  Launcher:");
				for (Pack p: list.launcher) {
					installPack(p);
				}
				System.out.println("  Packs:");
				for (Pack p: list.packs) {
					installPack(p);
				}
				System.out.println("  Extras:");
				for (Pack p: list.extras) {
					installPack(p);
				}
			
		} catch (Exception e) {
			printError("Unable to load packlist \"" + packlist + "\".", 4, e);
		}
	} // processList
	
	/**
	 * Install pack.
	 * @param	p Pack object created by List object.
	 */
	private static void installPack(Pack p) {
		
		// Setup
		System.out.println("  - Pack: " + p.pack + " (" + p.name + ")");
		boolean prompt = true;
		boolean install = false;
		String reply;
		
		// Prompt
		do {
			// Ask
			System.out.print("    Install (y/n/i)? ");

			// Auto-install
			if (auto) {
				System.out.println("y");
				install = true;
				prompt = false;
			// Get reply
			} else {
				reply = in.nextLine();
				if (reply.equals("y") || reply.equals("yes")) {
					install = true;
					prompt = false;
				} else if (reply.equals("n") || reply.equals("no")) {
					install = false;
					prompt = false;
				} else if (reply.equals("i") || reply.equals("info")) {
					System.out.println(
						"      Infomation" +
						"\n      - Pack Type: " + p.type +
						"\n      - Name: " + p.name + 
						"\n      - Authors: " + p.author +
						"\n      - Category: " + p.category +
						"\n      - Version: " + p.version +
						"\n      - Description: " + p.description +
						"\n      - Download: " + p.download +
						"\n      - Extract to: " + p.path
					);
				}
			}
		} while (prompt); // Prompt
		
		// Install
		if (install) {

			// Paths and locations
			String downloadURL = p.download;
			String downloadPath = DOWNLOAD_FOLDER + "/" + Downloader.getBaseName(p.list)
				+ "/" + p.type + "/" + p.pack;
			String downloadFile = downloadPath + "/" + Downloader.getFileName(downloadURL);
			String extractPath = ipod + p.path;
			
			// Download n Extract
			try {
				
				// Download
				System.out.print("    Downloading... ");
				Downloader.download(downloadURL, downloadPath);
				System.out.println("Done.");
				
				// Check Md5
				String md5 = Downloader.getMd5(downloadFile);
				if (p.Md5 == null || p.Md5.equals("") || !p.Md5.equalsIgnoreCase(md5)) {
					System.out.println("p.Md5: " + p.Md5 + " md5: " + md5);
					throw new Exception("Failed Md5 hash check.");
				}
				
				// Extract
				System.out.print("    Extracting... ");
				if (!Extractor.extract(downloadFile, extractPath)) {
					throw new Exception("Unsupported archive format.");
				}
				System.out.println("Done.");
				
			// Error!	
			} catch (Exception e) {
				printError("Unable to install pack \"" + p.pack + "\".", -1, e);
			}
			
			
			/*
			// Download location
			String download = p.download;
			String path = DOWNLOAD_FOLDER + "/" + p.list.substring(0,p.list.indexOf('.'))
				+ "/" + p.pack + "/";
			
			// Download and Extract
			try {
				// Download
				Downloader.download(download, path);
				
				// Extract
				Extractor.extract(path + Downloader.getName(download), ipod + p.path.substring(1));
			
				System.out.println("    Done.");
			} catch (Exception e) {
				Tools.printError("Unable to install pack \"" + p.pack + "\".", -1, e);
			}
			*/
			
		} // Install
		
	} // installPack
	
} // Main class
