// Package
package zeroslackr.installer;

// Imports
import java.io.*;

/**
 * Abstract Tools Class.
 * <p>
 * Collection of useful methods.
 * <p>
 * Licensed under <A HREF="http://www.gnu.org/licenses/gpl.txt">GNU GPL (Version 3)</A>. 
 * <p>
 * @author	Keripo
 * @version	Jan 18, 2008
 */

public abstract class Tools {

	/**
	 * Convert processed path to native equivalent.
	 * @param	path	String of path to be converted.
	 * @return	String of converted path.
	 */
	public static String convertPath(String path) {
		return path.replace('/', File.separatorChar);
	} // convertPath
	
	/**
	 * Get operating system of host computer.
	 * @return
	 * 		-1 if Undeterminable, <br>
	 *		 0 if Unsupported, <br>
	 * 		11 if Linux (86), <br>
	 * 		12 if Linux (64), <br>
	 * 		13 if Linux (PPC), <br>
	 * 		21 if Windows, <br>
	 * 		31 if Mac OSX
	 */
	public static int getHostOS() {
		
		// Operating System
		String bufferOS = System.getProperty("os.name");
		
		// Undeterminable
		if (bufferOS == null) {
			return -1;
		} else
	
		// Linux
		if (bufferOS.indexOf("Linux")!=-1) {
			// Architechture
			String bufferArch = System.getProperty("os.arch");
			if (bufferArch.indexOf("ppc")!=-1) {
				return 13;
			} else if (bufferArch.indexOf("64")!=-1) {
				return 12;
			} else if (bufferArch.indexOf("86")!=-1) {
				return 11;
			} else {
				return 11; // Hope will work
			}
		} else
			
		// Windows
		if (bufferOS.indexOf("Windows")!=-1) {
			return 21;
		} else
		
		// Mac OS X	
		if (bufferOS.indexOf("Mac OSX")!=-1) {
			return 31;
			
		// Unsupported
		} else {
			return 0;
		}
		
	} // getHostOS
	
	/**
	 * Get iPod's location.
	 * @return	Location of iPod. 
	 */
	public static String getiPodPath() throws Exception {
		switch (getHostOS()) {
			// Bad host
			case -1:
				throw new Exception("Unable to determine host operating system.");
			case  0:
				throw new Exception("Unsupported host operating system.");
			// Linux
			case 11:
			case 12:
			case 13:
				// TODO: return blah
				// iPods always found as "/dev/sd@#" devices
				// but needs to be mounted to "/mnt/ipod" for write access.
				// May manually ask user to specify location.
				throw new Exception("iPod not found.");
			// Windows
			case 21:
				for (int i = 65; i < 91; i++) { // letters A-Z
					if (new File((char)i + ":\\iPod_Control").exists()) {
						return (char)i + ":";
					}
				}
				throw new Exception("iPod not found.");
			// Mac OSX
			case 31:
				// TODO: return blah
				// iPods always found in "/Volume/" but location syntax unknown.
				// May manually ask user to specify location.
				throw new Exception("iPod not found.");
			// Just in case
			default:
				throw new Exception("iPod not found.");
		}
	} // getiPodPath

	
} // Tools class
